const save = require('./save');
const create = require('./create');

/**
 * Updates a model or creates the model if it cannot be found.
 * @param {string} table The name of the database table.
 * @param {APIBuilder.Model} Model The model class being updated.
 * @param {String} id ID of the model to update.
 * @param {Object} values Model attributes to set.
 * @returns {APIBuilder.Instance|null} Created item or null if update happened.
 * @throws {Error}
 */
module.exports = async function upsert(table, Model, id, values) {
	// Allow non-null falsy values as the pk
	if (id === null || id === undefined) {
		throw new Error('You must provide a value for the primary key.');
	}
	if (!values) {
		throw new Error('You must provide the values to be persisted.');
	}

	const primaryKeyColumn = this.getPrimaryKeyColumn(Model);
	if (!primaryKeyColumn) {
		throw new Error(`Can't find primary key column for ${table}`);
	}

	// Try to update
	const instance = Model.instance(values, false);
	instance.setPrimaryKey(id);

	const result = await save.call(this, table, Model, instance);

	// Update succeeded
	if (result) {
		this.logger.debug('upsert updated record', result);
		return instance;
	}

	this.logger.debug('upsert inserting record');
	// didn't update so try to create
	const createValues = {
		...values,
		[primaryKeyColumn]: id
	};
	return create.call(this, table, Model, createValues);
};
