const APIBuilder = require('@axway/api-builder-runtime');

/**
 * Creates models from the schema returned from `fetchSchema`. A utility method
 * `convertColumnToField` is required.
 * @example
 * {
 *     tables: {
 *         person: {
 *             info: {
 *                 schema: 'dbo'
 *                 name: 'person'
 *             },
 *             columns: {
 *                 name: {
 *                     TABLE_SCHEMA: 'dbo',
 *                     TABLE_NAME: 'tbl_user',
 *                     IS_NULLABLE: 'NO',
 *                     DATA_TYPE: 'varchar'
 *                 }
 *             },
 *             primaryKey: {
 *                 columnName: 'name',
 *                 autogenerated: false
 *             }
 *         }
 *     }
 * }
 */
module.exports = function createModelsFromSchema () {
	const models = {};

	for (const tableKey in this.schema.tables) {
		const table = this.schema.tables[tableKey];

		const modelInfo = {
			name: `${this.name}/${table.info.name}`,
			autogen: !!this.config.modelAutogen,
			fields: {},
			connector: this,
			generated: true,
			metadata: {
				table: table.info.name,
				schema: table.info.schema,
				primarykey: null // no pk
			}
		};

		for (const columnName in table.columns) {
			if (table.primaryKey.columnName === columnName) {
				// exclude the PK from the fields definition
				continue;
			}
			const column = table.columns[columnName];
			modelInfo.fields[columnName] = this.convertColumnToField(column);
		}

		if (table.primaryKey.columnName) {
			const { columnName, autogenerated } = table.primaryKey;
			const column = table.columns[columnName];
			const field = this.convertColumnToField(column);

			modelInfo.metadata.primarykey = columnName;
			modelInfo.metadata.primaryKeyDetails = {
				autogenerated,
				type: field.type.name.toLowerCase()
			};
		}

		models[modelInfo.name]
			= APIBuilder.Model.extend(modelInfo.name, modelInfo);
	}

	this.models = {
		...models,
		...this.models
	};
};
