/**
 * Add orderby to the query.
 * @param {object} query The query builder.
 * @param {object} params The params
 * @param {number} params.Model The Model.
 * @param {number} params.primaryKeyColumn The primary key.
 * @param {number} params.modelFields The fields of the model.
 * @param {number} params.order The order options.
 * @param {number} params.skip The skip options.
 * @returns {object} The query builder.
 */
module.exports = function buildOrder(query,
	{ Model, primaryKeyColumn, modelFields, order, skip, distinct }) {
	if (!order && !skip) {
		return query;
	}

	if (typeof order === 'string') {
		order = order.split(',')
			.filter(prop => prop.trim().length > 0)
			.reduce((res, prop) => {
				res[prop.trim()] = 1;
				return res;
			}, {});
	}

	order = Model.translateKeysForPayload(order);
	const orderKeys = order && Object.keys(order);
	if (orderKeys && orderKeys.length) {
		const orderFields = orderKeys.map(name => {
			const dir = order[name] === 1
				|| order[name] === true
				|| order[name] === '1'
				|| order[name] === 'true';

			return [ name, dir ? 'ASC' : 'DESC' ];
		});

		orderFields.forEach(([ name, dir ]) => {
			query.orderBy(name, dir);
		});
	} else if (skip) {
		// they are paginating with `skip` and have no `order` provided, but an order is required
		if (distinct) {
			// if they are using `distinct` we must order by that, otherwise, we get error:
			// `ORDER BY items must appear in the select list if SELECT DISTINCT is specified.`
			query.orderBy(distinct, 'ASC');
		} else if (primaryKeyColumn) {
			// if not distinct, and have a pk, then use that
			query.orderBy(primaryKeyColumn, 'ASC');
		} else {
			// otherwise, use the first field
			query.orderBy(modelFields[0], 'ASC');
		}
	}

	return query;
};
