/**
 * Add where filter to the query.
 * @param {object} query The query builder.
 * @param {object} params The params
 * @param {number} params.where The where options
 * @param {number} params.modelFields The Model fields.
 * @param {number} params.primaryKeyColumn The primary key field.
 * @returns {object} The query builder.
 */
module.exports = function buildWhere(query, { modelFields, primaryKeyColumn, where }) {
	if (!where) {
		return query;
	}

	// Error when providing fields that don't exist on the model
	for (const field in where) {
		if (field !== primaryKeyColumn && !modelFields.includes(field)) {
			throw new Error(`Unknown field ${field}`);
		}
	}

	for (const key in where) {
		if (where[key] === undefined) {
			continue;
		}
		if (where[key] === null) {
			query.whereNull(key);
			continue;
		}

		if (where[key].hasOwnProperty('$like')) {
			query.where(key, 'like', where[key].$like);
		} else if (where[key].hasOwnProperty('$lt')) {
			query.where(key, '<', where[key].$lt);
		} else if (where[key].hasOwnProperty('$lte')) {
			query.where(key, '<=', where[key].$lte);
		} else if (where[key].hasOwnProperty('$gt')) {
			query.where(key, '>', where[key].$gt);
		} else if (where[key].hasOwnProperty('$gte')) {
			query.where(key, '>=', where[key].$gte);
		} else if (where[key].hasOwnProperty('$ne')) {
			if (where[key].$ne === null) {
				query.whereNotNull(key);
			} else {
				query.where(key, '!=', where[key].$ne);
			}
		} else if (where[key].hasOwnProperty('$in')) {
			const val = Array.isArray(where[key].$in) ? where[key].$in : [ where[key].$in ];
			query.whereIn(key, val);
		} else if (where[key].hasOwnProperty('$nin')) {
			const val = Array.isArray(where[key].$nin) ? where[key].$nin : [ where[key].$nin ];
			query.whereNotIn(key, val);
		} else if (where[key].hasOwnProperty('$eq')) {
			if (where[key].$eq === null) {
				query.whereNull(key);
			} else {
				query.where(key, where[key].$eq);
			}
		} else {
			// we are relying on tedious to do validation on `where[key]` (as it
			// could be literally anything).
			query.where(key, where[key]);
		}
	}
	return query;
};
